import QtQuick 1.0

Rectangle {
    id: memory
    color: "darkgreen"

    // Configurable
    property int startSizeRows: 2 // Starting rows count
    property int startSizeCols: 3 // Starting cols count
    property int totalLevels: 3   // Number of levels in the game

    // Non-Configurable
    property int background: Math.floor(Math.random() * 4)
    property int sizeRows: 0
    property int sizeCols: 0
    property int selectionIndexA: -1
    property int selectionIndexB: -1
    property int currentLevel: 0
    property bool playing: false  // Either playing or displaying
    property bool initialized: false // Wether the board has been randomized
    property int moves: 0
    property int bestMoves: 100
    property int clock: 0

    width: 20 + 120 * (startSizeCols + totalLevels)
    height: 20 + 120 * (startSizeRows + totalLevels) + 20 // Additional 20 for top bar

    function gameInit()
    {
        var i;
        for (i = 0; i < sizeRows * sizeCols; i++)
        {
            var randIndex = Math.floor(Math.random() * (sizeRows * sizeCols));
            var saveFace = container.children[randIndex].face;
            container.children[randIndex].face = container.children[i].face;
            container.children[i].face = saveFace;
        }
    }

    function checkEndGame()
    {
        var i;
        var endGame = true;
        for (i = 0; i < sizeRows * sizeCols; i++) {
            if (!container.children[i].flipped) {
                endGame = false;
                break;
            }
        }
        if (endGame) {
            if (moves < bestMoves) bestMoves = moves;
            moves = 0;
            clockTimer.stop();
            if (currentLevel < totalLevels) {
                container.visible = false;
                playing = false;
                currentLevel++;
                levelTimer.start();
            }
        }
    }

    function checkGame()
    {
        if (container.children[selectionIndexA].face
                == container.children[selectionIndexB].face) {
            container.children[selectionIndexA].opacity = 0.3
            container.children[selectionIndexB].opacity = 0.3
        } else {
            container.children[selectionIndexA].flipped = false
            container.children[selectionIndexB].flipped = false
        }
        selectionIndexA = -1;
        selectionIndexB = -1;
        checkEndGame();
    }

    Timer {
        id: clockTimer
        running: false
        repeat: true
        interval: 1000
        onTriggered: clock++
    }

    Timer {
        id: checkTimer
        interval: 1000
        onTriggered: checkGame()
    }

    Timer {
        id: levelTimer
        interval: 2000
        repeat: false
        running:  true
        onTriggered: {playing = true; clock = 0; clockTimer.start()}
    }

    Rectangle {
        id: statusbar
        color: "#cccccc"
        radius: 5
        width: memory.width - 20
        height:  20
        anchors.top: parent.top
        anchors.left: parent.left
        anchors.topMargin: 10
        anchors.leftMargin: 10

        Flow {
            anchors.top: parent.top
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.topMargin: 0
            anchors.leftMargin: 0
            spacing: 40
            Text {font.pointSize: 12; text: "Moves: " + moves}
            Text {font.pointSize: 12; text: "Best Moves: " + bestMoves}
            Flow {
                spacing: 5
                Text {font.pointSize: 12; text: "Time: "}
                Text {font.pointSize: 12; font.family: "monospace"; text: Math.floor(clock / 60) + ":" + clock % 60}
            }
        }
    }

    Rectangle {
        anchors.top: statusbar.bottom
        anchors.bottom: parent.bottom
        anchors.horizontalCenter: parent.horizontalCenter

        Text {
            id: levelText
            text: "Level  "
            visible: true
            color: "white"
            anchors.verticalCenter: parent.verticalCenter
            anchors.horizontalCenter: parent.horizontalCenter
        }


        Grid {
            id: container
            visible: false
            rows: memory.sizeRows
            columns: memory.sizeCols
            spacing: 10
            anchors.verticalCenter: parent.verticalCenter
            anchors.horizontalCenter: parent.horizontalCenter

            Repeater {
                model: memory.sizeCols * memory.sizeRows
                Card {
                    background: memory.background
                    face: index % ((memory.sizeCols * memory.sizeRows)/2)
                    NumberAnimation on opacity {
                        duration: 300 + Math.random() * 500
                        from: 0; to: 1; easing.type: Easing.InCubic
                    }
                    onSelected: {
                        if (!initialized) {
                            gameInit();
                            initialized = true;
                        }
                        if (selectionIndexA >= 0 && selectionIndexB >= 0) {
                            checkTimer.stop();
                            checkGame();
                        }
                        if (selectionIndexA < 0) {
                            selectionIndexA = index;
                        } else {
                            selectionIndexB = index;
                            checkTimer.start();
                        }
                        moves++;
                    }
                }
            }
        }
    }

    states: [
        State {
            name: "level"
            when: !playing
            PropertyChanges {target: container; visible: false}
            PropertyChanges {
                target: levelText
                text: "Level " + (currentLevel + 1)
                visible: true
            }
        },
        State {
            name: "play"
            when: playing
            PropertyChanges {
                target: memory
                sizeRows: startSizeRows + currentLevel
                sizeCols: startSizeCols + currentLevel
                initialized: false
            }
            PropertyChanges {target: container; visible: true}
            PropertyChanges {target: levelText; visible: false}
        }
    ]

    transitions: [
        Transition {
            to: "level"
            NumberAnimation {
                target: levelText; property: "font.pointSize"
                from: 1; to: 100; duration: 300
                easing.type: Easing.InCubic
            }
        },
        Transition {
            to: "play"
            NumberAnimation {
                target: container; property: "opacity";
                from: 0; to: 1; duration: 500
                easing.type: Easing.InCubic
            }
        }
    ]
}
